@@ -8,8 +8,9 @@ jq "setpath([\"name\"]; \"fend-wasm-nodejs\")" ../wasm/pkg-nodejs/package.json >
 mv temp ../wasm/pkg-nodejs/package.json
 
 npm install
+npm exec tsc
+npm exec -- esbuild --bundle index.ts --outdir=dist --platform=node
 
 rm -f lambda_package.zip
-zip -r lambda_package.zip \
-	node_modules/ index.js \
-	package.json package-lock.json
+# don't include package.json because esbuild makes a CJS bundle that won't work if node finds `"type": "module"`
+zip -j lambda_package.zip dist/index.js node_modules/fend-wasm-nodejs/fend_wasm_bg.wasm
